try:
    from collections import OrderedDict
except ImportError:
    from django.utils.datastructures import SortedDict as OrderedDict  # noqa

try:
    from formtools.wizard.views import SessionWizardView
except ImportError:
    from django.contrib.formtools.wizard.views import SessionWizardView  # noqa

try:
    from cms.utils.plugins import build_plugin_tree
except ImportError:
    from cms.utils.plugins import get_plugins_as_layered_tree as build_plugin_tree  # noqa



from tablib import Dataset


class Exporter(object):

    def __init__(self, queryset):
        self.queryset = queryset

    def get_dataset(self, fields):
        headers = [field.rpartition('-')[0] for field in fields]
        dataset = Dataset(headers=headers)

        for submission in self.queryset.only('data').iterator():
            row_data = []
            form_fields = [field for field in submission.get_form_data()
                           if field.field_id in fields]

            for header in fields:
                for field in form_fields:
                    if field.field_id == header:
                        row_data.append(field.value)
                        break
                else:
                    row_data.append('')

            if row_data:
                dataset.append(row_data)
        return dataset

    def get_fields_for_export(self):
        old_fields = []
        old_field_ids = []

        # A user can add fields to the form over time,
        # knowing this we use the latest form submission as a way
        # to get the latest form state.
        submissions = self.queryset.only('data').iterator()

        latest_data = next(submissions)
        latest_fields = [field for field in latest_data.get_form_data()
                         if field.label]
        latest_field_ids = [field.field_id for field in latest_fields]

        for submission in submissions:
            fields = submission.get_form_data()

            for field in fields:
                if not field.label:
                    continue

                field_id = field.field_id

                if (field_id not in old_field_ids) and (field_id not in latest_field_ids):
                    old_fields.append(field)
                    old_field_ids.append(field_id)
        return (latest_fields, old_fields)



# Generated by Django 2.2.14 on 2020-07-15 14:05

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0004_auto_20190104_1242'),
    ]

    operations = [
        migrations.AddField(
            model_name='emailnotification',
            name='reply_to_email',
            field=models.CharField(blank=True, max_length=200, verbose_name='reply to email'),
        ),
    ]



# Generated by Django 1.9.8 on 2016-08-21 14:26
from __future__ import unicode_literals

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0004_auto_20151207_1825'),
    ]

    operations = [
        migrations.DeleteModel(
            name='FormData',
        ),
        migrations.AlterModelOptions(
            name='formsubmission',
            options={'ordering': ['-sent_at'], 'verbose_name': 'Form submission', 'verbose_name_plural': 'Form submissions'},
        ),
        migrations.AlterField(
            model_name='emailfieldplugin',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Field is required'),
        ),
        migrations.AlterField(
            model_name='fieldplugin',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Field is required'),
        ),
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Field is required'),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Field is required'),
        ),
        migrations.AlterField(
            model_name='textareafieldplugin',
            name='required',
            field=models.BooleanField(default=False, verbose_name='Field is required'),
        ),
    ]



import os
from string import Template

from emailit.utils import get_template_name


EMAIL_TEMPLATES_BASE = 'aldryn_forms/email_notifications/emails/'

EMAIL_THEMES_PATH = os.path.join(EMAIL_TEMPLATES_BASE, 'themes')
EMAIL_NOTIFICATIONS_PATH = os.path.join(EMAIL_TEMPLATES_BASE, 'notification')


def get_email_template_name(name='body', suffix='txt'):
    template_name = get_template_name(
        # we don't care about language specific templates
        language=None,
        base=EMAIL_NOTIFICATIONS_PATH,
        part=name,
        suffix=suffix,
    )
    return template_name


def get_theme_template_name(theme, suffix='txt'):
    filename = '{0}.{1}'.format(theme, suffix)
    template_name = os.path.join(EMAIL_THEMES_PATH, filename)
    return template_name


def render_text(message, context):
    template = Template(template=message)
    return template.safe_substitute(**context)



# Generated by Django 2.2.13 on 2022-03-14 15:12

import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields
import filer.fields.folder

import aldryn_forms.models
import aldryn_forms.sizefield.models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0013_add_field_is_enable_autofill_from_url_params'),
    ]

    operations = [
        migrations.AddField(
            model_name='fileuploadfieldplugin',
            name='accepted_types',
            field=models.CharField(blank=True, help_text='The list of accepted types. E.g. ".pdf .jpg .png text/plain application/msword image/*".', max_length=255, null=True, validators=[aldryn_forms.models.validate_accepted_types], verbose_name='Accepted types'),
        ),
        migrations.AddField(
            model_name='fileuploadfieldplugin',
            name='enable_js',
            field=models.NullBooleanField(help_text='Enable javascript to view files for upload.', verbose_name='Enable js'),
        ),
        migrations.AddField(
            model_name='imageuploadfieldplugin',
            name='enable_js',
            field=models.NullBooleanField(help_text='Enable javascript to view files for upload.', verbose_name='Enable js'),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='action_backend',
            field=models.CharField(choices=[('default', 'Default'), ('email_only', 'Email only'), ('none', 'None')], default='default', max_length=15, verbose_name='Action backend'),
        ),
        migrations.CreateModel(
            name='MultipleFilesUploadFieldPlugin',
            fields=[
                ('name', models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name')),
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('required', models.BooleanField(default=False, verbose_name='Field is required')),
                ('required_message', models.TextField(blank=True, help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message')),
                ('placeholder_text', models.CharField(blank=True, help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"', max_length=255, verbose_name='Placeholder text')),
                ('help_text', models.TextField(blank=True, help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('min_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Min value')),
                ('max_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value')),
                ('initial_value', models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value')),
                ('custom_classes', models.CharField(blank=True, max_length=255, verbose_name='custom css classes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_multiplefilesuploadfieldplugin', serialize=False, to='cms.CMSPlugin')),
                ('max_size', aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum file size of the upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum file size')),
                ('enable_js', models.NullBooleanField(help_text='Enable javascript to view files for upload.', verbose_name='Enable js')),
                ('accepted_types', models.CharField(blank=True, help_text='The list of accepted types. E.g. ".pdf .jpg .png text/plain application/msword image/*".', max_length=255, null=True, validators=[aldryn_forms.models.validate_accepted_types], verbose_name='Accepted types')),
                ('max_files', models.PositiveSmallIntegerField(blank=True, help_text='Maximum number of uploaded files.', null=True, verbose_name='Max. files')),
                ('upload_to', filer.fields.folder.FilerFolderField(help_text='Select a folder to which all files submitted through this field will be uploaded to.', on_delete=django.db.models.deletion.CASCADE, to='filer.Folder', verbose_name='Upload files to')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
    ]



import json
import re
import warnings
from collections import defaultdict, namedtuple
from functools import partial
from typing import List

from django.conf import settings
from django.core.exceptions import ValidationError
from django.db import models
from django.db.models.functions import Coalesce
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _

from cms.cms_plugins import AliasPlugin
from cms.models.fields import PageField
from cms.models.pluginmodel import CMSPlugin
from cms.utils.plugins import downcast_plugins

from djangocms_attributes_field.fields import AttributesField
from filer.fields.folder import FilerFolderField

from .compat import build_plugin_tree
from .helpers import is_form_element
from .sizefield.models import FileSizeField
from .utils import ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE, action_backend_choices, get_action_backends


AUTH_USER_MODEL = getattr(settings, 'AUTH_USER_MODEL', 'auth.User')


# Once djangoCMS < 3.3.1 support is dropped
# Remove the explicit cmsplugin_ptr field declarations
CMSPluginField = partial(
    models.OneToOneField,
    to=CMSPlugin,
    related_name='%(app_label)s_%(class)s',
    parent_link=True,
)

FieldData = namedtuple(
    'FieldData',
    field_names=['label', 'value']
)
FormField = namedtuple(
    'FormField',
    field_names=[
        'name',
        'label',
        'plugin_instance',
        'field_occurrence',
        'field_type_occurrence',
    ]
)
Recipient = namedtuple(
    'Recipient',
    field_names=['name', 'email']
)
BaseSerializedFormField = namedtuple(
    'SerializedFormField',
    field_names=[
        'name',
        'label',
        'field_occurrence',
        'value',
    ]
)


class SerializedFormField(BaseSerializedFormField):

    # For _asdict() with Py3K
    __slots__ = ()

    @property
    def field_id(self):
        field_label = self.label.strip()

        if field_label:
            field_as_string = f'{field_label}-{self.field_type}'
        else:
            field_as_string = self.name
        field_id = f'{field_as_string}:{self.field_occurrence}'
        return field_id

    @property
    def field_type_occurrence(self):
        return self.name.rpartition('_')[1]

    @property
    def field_type(self):
        return self.name.rpartition('_')[0]


class BaseFormPlugin(CMSPlugin):
    FALLBACK_FORM_TEMPLATE = 'aldryn_forms/form.html'
    DEFAULT_FORM_TEMPLATE = getattr(
        settings, 'ALDRYN_FORMS_DEFAULT_TEMPLATE', FALLBACK_FORM_TEMPLATE)

    FORM_TEMPLATES = ((DEFAULT_FORM_TEMPLATE, _('Default')),)

    if hasattr(settings, 'ALDRYN_FORMS_TEMPLATES'):
        FORM_TEMPLATES += settings.ALDRYN_FORMS_TEMPLATES

    REDIRECT_TO_PAGE = 'redirect_to_page'
    REDIRECT_TO_URL = 'redirect_to_url'
    REDIRECT_CHOICES = [
        (REDIRECT_TO_PAGE, _('CMS Page')),
        (REDIRECT_TO_URL, _('Absolute URL')),
    ]

    _form_elements = None
    _form_field_key_cache = None

    name = models.CharField(
        verbose_name=_('Name'),
        max_length=255,
        help_text=_('Used to filter out form submissions.'),
    )
    error_message = models.TextField(
        verbose_name=_('Error message'),
        blank=True,
        null=True,
        help_text=_('An error message that will be displayed if the form '
                    'doesn\'t validate.')
    )
    success_message = models.TextField(
        verbose_name=_('Success message'),
        blank=True,
        null=True,
        help_text=_('An success message that will be displayed.')
    )
    redirect_type = models.CharField(
        verbose_name=_('Redirect to'),
        max_length=20,
        choices=REDIRECT_CHOICES,
        help_text=_('Where to redirect the user when the form has been successfully sent?'),
        blank=True,
    )
    url = models.URLField(_('Absolute URL'), blank=True, null=True)
    custom_classes = models.CharField(
        verbose_name=_('custom css classes'), max_length=255, blank=True)
    form_template = models.CharField(
        verbose_name=_('form template'),
        max_length=255,
        choices=FORM_TEMPLATES,
        default=DEFAULT_FORM_TEMPLATE,
    )

    # Staff notification email settings
    recipients = models.ManyToManyField(
        to=AUTH_USER_MODEL,
        verbose_name=_('Recipients'),
        blank=True,
        limit_choices_to={'is_staff': True},
        help_text=_('People who will get the form content via e-mail.')
    )

    action_backend = models.CharField(
        verbose_name=_('Action backend'),
        max_length=ALDRYN_FORMS_ACTION_BACKEND_KEY_MAX_SIZE,
        default='default',
        choices=action_backend_choices(),
    )

    form_attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
    )

    redirect_page = PageField(
        verbose_name=_('CMS Page'),
        blank=True,
        null=True,
        on_delete=models.SET_NULL,
    )

    is_enable_autofill_from_url_params = models.BooleanField(
        default=False,
        verbose_name=_("Enable autofill from url parameters"),
        help_text=_(
            "Eg if you open the form with a url that contains parameters as "
            "'https://example.com/sub-page/?email=hello@example.com&name=Alex', "
            "then the fields 'email' and 'name' are going to to be filled in automatically."
        )
    )

    cmsplugin_ptr = CMSPluginField(
        on_delete=models.CASCADE,
    )

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    @property
    def page(self):
        warnings.warn(
            'The "page" field has been renamed to redirect_page '
            'and will be removed on Aldryn Forms 3.1.0',
            PendingDeprecationWarning
        )
        return self.redirect_page

    @page.setter
    def page(self, value):
        warnings.warn(
            'The "page" field has been renamed to redirect_page '
            'and will be removed on Aldryn Forms 3.1.0',
            PendingDeprecationWarning
        )
        self.redirect_page = value

    @cached_property
    def success_url(self):
        if self.redirect_type == FormPlugin.REDIRECT_TO_PAGE:
            return self.redirect_page.get_absolute_url()
        elif self.redirect_type == FormPlugin.REDIRECT_TO_URL and self.url:
            return self.url

    def copy_relations(self, oldinstance):
        for recipient in oldinstance.recipients.all():
            self.recipients.add(recipient)

    def get_submit_button(self):
        from .cms_plugins import SubmitButton

        form_elements = self.get_form_elements()

        for element in form_elements:
            plugin_class = element.get_plugin_class()

            if issubclass(plugin_class, SubmitButton):
                return element
        return

    def get_form_fields(self) -> List[FormField]:
        from .cms_plugins import Field

        fields = []

        # A field occurrence is how many times does a field
        # with the same label and type appear within the same form.
        # This is used as an identifier for the field within multiple forms.
        field_occurrences = defaultdict(lambda: 1)

        # A field type occurrence is how many times does a field
        # with the same type appear within the same form.
        # This is used as an identifier for the field within this form.
        field_type_occurrences = defaultdict(lambda: 1)

        form_elements = self.get_form_elements()
        field_plugins = []
        for plugin in form_elements:
            if issubclass(plugin.get_plugin_class(), Field):
                field_plugins.append(plugin)
            elif issubclass(plugin.get_plugin_class(), AliasPlugin) and \
                    issubclass(plugin.plugin.get_plugin_class(), Field):
                field_plugins.append(plugin.plugin.get_plugin_instance()[0])

        unique_field_names = []
        for field_plugin in field_plugins:
            field_type = field_plugin.field_type

            if field_type in field_type_occurrences:
                field_type_occurrences[field_type] += 1

            field_label = field_plugin.get_label()
            field_type_occurrence = field_type_occurrences[field_type]

            if field_plugin.name:
                field_name = field_plugin.name
            else:
                field_name = f'{field_type}_{field_type_occurrence}'

            if field_label:
                field_id = f'{field_type}_{field_label}'
            else:
                field_id = field_name

            if field_id in field_occurrences:
                field_occurrences[field_id] += 1

            # Make filed names unique.
            while field_name in unique_field_names:
                field_name += "_"
            unique_field_names.append(field_name)

            field = FormField(
                name=field_name,
                label=field_label,
                plugin_instance=field_plugin,
                field_occurrence=field_occurrences[field_id],
                field_type_occurrence=field_type_occurrence,
            )
            fields.append(field)
        return fields

    def get_form_field_name(self, field: 'FieldPluginBase') -> str:
        if self._form_field_key_cache is None:
            self._form_field_key_cache = {}

        is_cache_needs_update = field.pk not in self._form_field_key_cache
        if is_cache_needs_update:
            form_fields: List[FormField] = self.get_form_fields()
            for form_field in form_fields:
                self._form_field_key_cache[form_field.plugin_instance.pk] = form_field.name

        return self._form_field_key_cache[field.pk]

    def get_form_fields_as_choices(self):
        fields = self.get_form_fields()

        for field in fields:
            yield (field.name, field.label)

    def get_form_elements(self):
        from .utils import get_nested_plugins

        if self.child_plugin_instances is None:
            descendants = self.get_descendants().order_by('path')
            # Set parent_id to None in order to
            # fool the build_plugin_tree function.
            # This is sadly necessary to avoid getting all nodes
            # higher than the form.
            parent_id = self.parent_id
            self.parent_id = None
            # Important that this is a list in order to modify
            # the current instance
            descendants_with_self = [self] + list(descendants)
            # Let the cms build the tree
            build_plugin_tree(descendants_with_self)
            # Set back the original parent
            self.parent_id = parent_id

        if self._form_elements is None:
            children = get_nested_plugins(self)
            children_instances = downcast_plugins(children)
            self._form_elements = [
                p for p in children_instances if is_form_element(p)]
        return self._form_elements


class FormPlugin(BaseFormPlugin):

    class Meta:
        abstract = False

    def __str__(self):
        return self.name


class FieldsetPlugin(CMSPlugin):

    legend = models.CharField(_('Legend'), max_length=255, blank=True)
    custom_classes = models.CharField(
        verbose_name=_('custom css classes'), max_length=255, blank=True)
    cmsplugin_ptr = CMSPluginField(
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return self.legend or str(self.pk)


class FieldPluginBase(CMSPlugin):
    name = models.CharField(
        _('Name'),
        max_length=255,
        help_text=_('Used to set the field name'),
        blank=True,
    )
    label = models.CharField(_('Label'), max_length=255, blank=True)
    required = models.BooleanField(_('Field is required'), default=False)
    required_message = models.TextField(
        verbose_name=_('Error message'),
        blank=True,
        null=True,
        help_text=_('Error message displayed if the required field is left '
                    'empty. Default: "This field is required".')
    )
    placeholder_text = models.CharField(
        verbose_name=_('Placeholder text'),
        max_length=255,
        blank=True,
        help_text=_('Default text in a form. Disappears when user starts '
                    'typing. Example: "email@example.com"')
    )
    help_text = models.TextField(
        verbose_name=_('Help text'),
        blank=True,
        null=True,
        help_text=_('Explanatory text displayed next to input field. Just like '
                    'this one.')
    )
    attributes = AttributesField(
        verbose_name=_('Attributes'),
        blank=True,
        excluded_keys=['name']
    )

    # for text field those are min and max length
    # for multiple select those are min and max number of choices
    min_value = models.PositiveIntegerField(
        _('Min value'),
        blank=True,
        null=True,
    )

    max_value = models.PositiveIntegerField(
        _('Max value'),
        blank=True,
        null=True,
    )
    initial_value = models.CharField(
        verbose_name=_('Initial value'),
        max_length=255,
        blank=True,
        help_text=_('Default value of field.')
    )

    custom_classes = models.CharField(
        verbose_name=_('custom css classes'), max_length=255, blank=True)
    cmsplugin_ptr = CMSPluginField(
        on_delete=models.CASCADE,
    )

    class Meta:
        abstract = True

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        if self.plugin_type:
            attribute = 'is_%s' % self.field_type
            setattr(self, attribute, True)

    def __str__(self):
        return self.label or self.name or str(self.pk)

    @property
    def field_type(self):
        return self.plugin_type.lower()

    def get_label(self):
        return self.label or self.placeholder_text

    def clean(self):
        if ' ' in self.name:
            raise ValidationError(_('The "name" field cannot contain spaces.'))


class FieldPlugin(FieldPluginBase):
    def copy_relations(self, oldinstance):
        for option in oldinstance.option_set.all():
            option.pk = None  # copy on save
            option.field = self
            option.save()


class TextAreaFieldPlugin(FieldPluginBase):
    text_area_columns = models.PositiveIntegerField(
        verbose_name=_('columns'), blank=True, null=True)
    text_area_rows = models.PositiveIntegerField(
        verbose_name=_('rows'), blank=True, null=True)


class EmailFieldPlugin(FieldPluginBase):
    email_send_notification = models.BooleanField(
        verbose_name=_('send notification when form is submitted'),
        default=False,
        help_text=_('When checked, the value of this field will be used to '
                    'send an email notification.')
    )
    email_subject = models.CharField(
        verbose_name=_('email subject'),
        max_length=255,
        blank=True,
        default='',
        help_text=_('Used as the email subject when email_send_notification '
                    'is checked.')
    )
    email_body = models.TextField(
        verbose_name=_('Additional email body'),
        blank=True,
        default='',
        help_text=_('Additional body text used when email notifications '
                    'are active.')
    )

    def get_parent_form(self):
        parent = self.get_parent()
        while parent is not None:
            if parent.plugin_type in ("FormPlugin", "EmailNotificationForm"):
                break
            parent = parent.get_parent()
        return parent

    def get_parent_form_action_backend(self):
        parent = self.get_parent_form()
        if parent is not None:
            return parent, get_action_backends().get(parent.get_plugin_instance()[0].action_backend)
        return None, None


class FileFieldPluginBase(FieldPluginBase):
    upload_to = FilerFolderField(
        verbose_name=_('Upload files to'),
        help_text=_('Select a folder to which all files submitted through '
                    'this field will be uploaded to.'),
        on_delete=models.CASCADE,
    )
    max_size = FileSizeField(
        verbose_name=_('Maximum files size'),
        null=True, blank=True,
        help_text=_('The maximum size of all files to upload, in bytes. You can '
                    'use common size suffixes (kB, MB, GB, ...).')
    )
    enable_js = models.BooleanField(
        verbose_name=_('Enable js'),
        null=True, blank=True,
        help_text=_('Enable javascript to view files for upload.')
    )

    class Meta:
        abstract = True


def validate_accepted_types(value):
    """Validate accepted types."""
    for code in re.split(r"\s+", value):
        if code[:1] == '.':
            if not re.match(r'\.\w+', code):
                raise ValidationError(_('%(value)s is not extension.'), params={'value': value})
        else:
            if not re.match(r'\w+/(\w+|\*)', code):
                raise ValidationError(_('%(value)s is not mimetype.'), params={'value': value})


class FileUploadFieldPlugin(FileFieldPluginBase):
    accepted_types = models.CharField(
        verbose_name=_('Accepted types'),
        max_length=255, null=True, blank=True,
        help_text=_('The list of accepted types. E.g. ".pdf .jpg .png text/plain application/msword image/*".'),
        validators=[validate_accepted_types]
    )


class MultipleFilesUploadFieldPlugin(FileFieldPluginBase):
    accepted_types = models.CharField(
        verbose_name=_('Accepted types'),
        max_length=255, null=True, blank=True,
        help_text=_('The list of accepted types. E.g. ".pdf .jpg .png text/plain application/msword image/*".'),
        validators=[validate_accepted_types]
    )
    max_files = models.PositiveSmallIntegerField(
        verbose_name=_('Max. files'),
        null=True, blank=True,
        help_text=_('Maximum number of uploaded files.'),
    )


class ImageUploadFieldPlugin(FileFieldPluginBase):
    max_width = models.PositiveIntegerField(
        verbose_name=_('Maximum image width'),
        null=True, blank=True,
        help_text=_('The maximum width of the uploaded image, in pixels.')
    )
    max_height = models.PositiveIntegerField(
        verbose_name=_('Maximum image height'),
        null=True, blank=True,
        help_text=_('The maximum height of the uploaded image, in pixels.')
    )


class DateFieldPlugin(FieldPluginBase):
    """Date field plugin."""
    earliest_date = models.DateField(
        verbose_name=_('Earliest date'),
        blank=None, null=True,
        help_text=_('The earliest date to accept.'))
    latest_date = models.DateField(
        verbose_name=_('Latest date'),
        blank=None, null=True,
        help_text=_('The latest date to accept.'))
    input_step = models.PositiveIntegerField(
        blank=True, null=True, verbose_name=_('Step'),
        help_text=_('The granularity numnber.'))


class DateTimeLocalFieldPlugin(FieldPluginBase):
    """Date time local field plugin."""
    earliest_datetime = models.DateTimeField(
        verbose_name=_('Earliest datetime'),
        blank=None, null=True,
        help_text=_('The earliest datetime to accept.'))
    latest_datetime = models.DateTimeField(
        verbose_name=_('Latest datetime'),
        blank=None, null=True,
        help_text=_('The latest datetime to accept.'))
    input_step = models.PositiveIntegerField(
        blank=True, null=True, verbose_name=_('Step'),
        help_text=_('The granularity numnber.'))


class TimeFieldPlugin(FieldPluginBase):
    """Time field plugin."""
    earliest_time = models.TimeField(
        verbose_name=_('Earliest time'),
        blank=None, null=True,
        help_text=_('The earliest time to accept.'))
    latest_time = models.TimeField(
        verbose_name=_('Latest time'),
        blank=None, null=True,
        help_text=_('The latest time to accept.'))
    input_step = models.PositiveIntegerField(
        blank=True, null=True, verbose_name=_('Step'),
        help_text=_('The granularity numnber.'))
    data_list = models.SlugField(
        blank=True, null=True, verbose_name=_('Datalist'),
        help_text=_('Datalist id of html element datalist with options.'))
    readonly = models.BooleanField(
        default=False, verbose_name=_('Read only'),
        help_text=_('The field cannot be edited by the user.'))


class Option(models.Model):
    field = models.ForeignKey(FieldPlugin, editable=False, on_delete=models.CASCADE)
    value = models.CharField(_('Value'), max_length=255)
    default_value = models.BooleanField(_('Default'), default=False)
    position = models.PositiveIntegerField(_('Position'), blank=True)

    class Meta:
        verbose_name = _('Option')
        verbose_name_plural = _('Options')
        ordering = ('position', )

    def __str__(self):
        return self.value

    def set_position(self):
        if self.position is None:
            self.position = self.field.option_set.aggregate(
                max_position=Coalesce(models.Max('position'), 0)
            ).get('max_position', 0) + 10

    def save(self, *args, **kwargs):
        self.set_position()
        return super().save(*args, **kwargs)


class FormButtonPlugin(CMSPlugin):
    label = models.CharField(_('Label'), max_length=255)
    custom_classes = models.CharField(
        verbose_name=_('custom css classes'), max_length=255, blank=True)
    cmsplugin_ptr = CMSPluginField(
        on_delete=models.CASCADE,
    )

    def __str__(self):
        return self.label


class FormSubmissionBase(models.Model):

    name = models.CharField(
        max_length=255,
        verbose_name=_('form name'),
        db_index=True,
        editable=False
    )

    data = models.TextField(blank=True, editable=False)
    recipients = models.TextField(
        verbose_name=_('users notified'),
        blank=True,
        help_text=_('People who got a notification when form was submitted.'),
        editable=False,
    )
    language = models.CharField(
        verbose_name=_('form language'),
        max_length=10,
        choices=settings.LANGUAGES,
        default=settings.LANGUAGE_CODE
    )
    form_url = models.CharField(
        verbose_name=_('form url'),
        max_length=255,
        blank=True,
    )
    sent_at = models.DateTimeField(auto_now_add=True)

    class Meta:
        abstract = True

    def __str__(self):
        return self.name

    def _form_data_hook(self, data, occurrences):
        field_label = data['label'].strip()

        if field_label:
            field_type = data['name'].rpartition('_')[0]
            field_id = f'{field_type}_{field_label}'
        else:
            field_id = data['name']

        if field_id in occurrences:
            occurrences[field_id] += 1

        data['field_occurrence'] = occurrences[field_id]
        return SerializedFormField(**data)

    def _recipients_hook(self, data):
        return Recipient(**data)

    def get_form_data(self):
        occurrences = defaultdict(lambda: 1)

        data_hook = partial(self._form_data_hook, occurrences=occurrences)

        try:
            form_data = json.loads(
                self.data,
                object_hook=data_hook,
            )
        except ValueError:
            # TODO: Log this?
            form_data = []
        return form_data

    def get_recipients(self):
        try:
            recipients = json.loads(
                self.recipients,
                object_hook=self._recipients_hook
            )
        except ValueError:
            # TODO: Log this?
            recipients = []
        return recipients

    def set_form_data(self, form):
        fields = form.get_serialized_fields(is_confirmation=False)
        fields_as_dicts = [field._asdict() for field in fields]

        self.data = json.dumps(fields_as_dicts)

    def set_recipients(self, recipients):
        raw_recipients = [
            {'name': rec[0], 'email': rec[1]} for rec in recipients]
        self.recipients = json.dumps(raw_recipients)


class FormSubmission(FormSubmissionBase):

    class Meta:
        ordering = ['-sent_at']
        verbose_name = _('Form submission')
        verbose_name_plural = _('Form submissions')



# Generated by Django 2.2.16 on 2020-09-15 16:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0012_auto_20190104_1242'),
    ]

    operations = [
        migrations.AddField(
            model_name='formplugin',
            name='is_enable_autofill_from_url_params',
            field=models.BooleanField(default=False, help_text="Eg if you open the form with a url that contains parameters as 'https://example.com/sub-page/?email=hello@example.com&name=Alex', then the fields 'email' and 'name' are going to to be filled in automatically.", verbose_name='Enable autofill from url parameters'),
        ),
    ]



# Generated by Django 1.10.8 on 2018-02-06 19:33
from __future__ import unicode_literals

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('email_notifications', '0002_form_plugin_add_missing_permission'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='emailnotification',
            options={'verbose_name': 'Email notification', 'verbose_name_plural': 'Email notifications'},
        ),
    ]



# Generated by Django 2.2.13 on 2022-03-18 08:16

from django.db import migrations

import aldryn_forms.sizefield.models


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0014_multiple_files_pload'),
    ]

    operations = [
        migrations.AlterField(
            model_name='fileuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
        migrations.AlterField(
            model_name='imageuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
        migrations.AlterField(
            model_name='multiplefilesuploadfieldplugin',
            name='max_size',
            field=aldryn_forms.sizefield.models.FileSizeField(blank=True, help_text='The maximum size of all files to upload, in bytes. You can use common size suffixes (kB, MB, GB, ...).', null=True, verbose_name='Maximum files size'),
        ),
    ]



# Generated by Django 2.2.13 on 2023-01-26 13:16

import django.db.models.deletion
from django.db import migrations, models

import djangocms_attributes_field.fields


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0015_fileupload_name_help_text'),
    ]

    operations = [
        migrations.CreateModel(
            name='DateFieldPlugin',
            fields=[
                ('name', models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name')),
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('required', models.BooleanField(default=False, verbose_name='Field is required')),
                ('required_message', models.TextField(blank=True, help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message')),
                ('placeholder_text', models.CharField(blank=True, help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"', max_length=255, verbose_name='Placeholder text')),
                ('help_text', models.TextField(blank=True, help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('min_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Min value')),
                ('max_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value')),
                ('initial_value', models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value')),
                ('custom_classes', models.CharField(blank=True, max_length=255, verbose_name='custom css classes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_datefieldplugin', serialize=False, to='cms.CMSPlugin')),
                ('earliest_date', models.DateField(blank=None, help_text='The earliest date to accept.', null=True, verbose_name='Earliest date')),
                ('latest_date', models.DateField(blank=None, help_text='The latest date to accept.', null=True, verbose_name='Latest date')),
                ('input_step', models.PositiveIntegerField(blank=True, help_text='The granularity numnber.', null=True, verbose_name='Step')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='DateTimeLocalFieldPlugin',
            fields=[
                ('name', models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name')),
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('required', models.BooleanField(default=False, verbose_name='Field is required')),
                ('required_message', models.TextField(blank=True, help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message')),
                ('placeholder_text', models.CharField(blank=True, help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"', max_length=255, verbose_name='Placeholder text')),
                ('help_text', models.TextField(blank=True, help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('min_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Min value')),
                ('max_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value')),
                ('initial_value', models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value')),
                ('custom_classes', models.CharField(blank=True, max_length=255, verbose_name='custom css classes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_datetimelocalfieldplugin', serialize=False, to='cms.CMSPlugin')),
                ('earliest_datetime', models.DateTimeField(blank=None, help_text='The earliest datetime to accept.', null=True, verbose_name='Earliest datetime')),
                ('latest_datetime', models.DateTimeField(blank=None, help_text='The latest datetime to accept.', null=True, verbose_name='Latest datetime')),
                ('input_step', models.PositiveIntegerField(blank=True, help_text='The granularity numnber.', null=True, verbose_name='Step')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.CreateModel(
            name='TimeFieldPlugin',
            fields=[
                ('name', models.CharField(blank=True, help_text='Used to set the field name', max_length=255, verbose_name='Name')),
                ('label', models.CharField(blank=True, max_length=255, verbose_name='Label')),
                ('required', models.BooleanField(default=False, verbose_name='Field is required')),
                ('required_message', models.TextField(blank=True, help_text='Error message displayed if the required field is left empty. Default: "This field is required".', null=True, verbose_name='Error message')),
                ('placeholder_text', models.CharField(blank=True, help_text='Default text in a form. Disappears when user starts typing. Example: "email@example.com"', max_length=255, verbose_name='Placeholder text')),
                ('help_text', models.TextField(blank=True, help_text='Explanatory text displayed next to input field. Just like this one.', null=True, verbose_name='Help text')),
                ('attributes', djangocms_attributes_field.fields.AttributesField(blank=True, default=dict, verbose_name='Attributes')),
                ('min_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Min value')),
                ('max_value', models.PositiveIntegerField(blank=True, null=True, verbose_name='Max value')),
                ('initial_value', models.CharField(blank=True, help_text='Default value of field.', max_length=255, verbose_name='Initial value')),
                ('custom_classes', models.CharField(blank=True, max_length=255, verbose_name='custom css classes')),
                ('cmsplugin_ptr', models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, parent_link=True, primary_key=True, related_name='aldryn_forms_timefieldplugin', serialize=False, to='cms.CMSPlugin')),
                ('earliest_time', models.TimeField(blank=None, help_text='The earliest time to accept.', null=True, verbose_name='Earliest time')),
                ('latest_time', models.TimeField(blank=None, help_text='The latest time to accept.', null=True, verbose_name='Latest time')),
                ('input_step', models.PositiveIntegerField(blank=True, help_text='The granularity numnber.', null=True, verbose_name='Step')),
                ('data_list', models.SlugField(blank=True, help_text='Datalist id of html element datalist with options.', null=True, verbose_name='Datalist')),
                ('readonly', models.BooleanField(default=False, help_text='The field cannot be edited by the user.', verbose_name='Read only')),
            ],
            options={
                'abstract': False,
            },
            bases=('cms.cmsplugin',),
        ),
        migrations.AlterField(
            model_name='formplugin',
            name='action_backend',
            field=models.CharField(choices=[('none', 'None'), ('email_only', 'Email only'), ('default', 'Default')], default='default', max_length=15, verbose_name='Action backend'),
        ),
    ]



from setuptools import find_packages, setup

from aldryn_forms import __version__


REQUIREMENTS = [
    'django-cms>=3.11',
    'django-emailit',
    'djangocms-text-ckeditor',
    'djangocms-attributes-field>=1.0.0',
    'django-tablib',
    'tablib',
    'django-filer',
    'django-sizefield',
    'markdown',
    'Pillow',
]


CLASSIFIERS = [
    'Development Status :: 5 - Production/Stable',
    'Environment :: Web Environment',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: BSD License',
    'Operating System :: OS Independent',
    'Framework :: Django',
    'Framework :: Django :: 3.2',
    'Framework :: Django :: 4.0',
    'Programming Language :: Python',
    'Programming Language :: Python :: 3',
    'Programming Language :: Python :: 3.7',
    'Programming Language :: Python :: 3.8',
    'Programming Language :: Python :: 3.9',
    'Programming Language :: Python :: 3.10',
    'Topic :: Internet :: WWW/HTTP',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    'Topic :: Software Development',
    'Topic :: Software Development :: Libraries',
]


setup(
    name='djangocms-aldryn-forms',
    version=__version__,
    author='Divio AG',
    author_email='info@divio.ch',
    url='https://github.com/CZ-NIC/djangocms-aldryn-forms',
    license='BSD',
    description='Create forms and embed them on CMS pages.',
    long_description=open('README.rst').read(),
    long_description_content_type='text/x-rst',
    packages=find_packages(exclude=['tests']),
    include_package_data=True,
    zip_safe=False,
    install_requires=REQUIREMENTS,
    extras_require={
        'captcha': ['django-simple-captcha'],
    },
    classifiers=CLASSIFIERS,
    test_suite='tests.settings.run',
)



# Generated by Django 1.10.8 on 2018-01-10 15:00
from __future__ import unicode_literals

from django.db import migrations


def forward_migration(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    FormPlugin = apps.get_model('aldryn_forms', 'FormPlugin')
    FormPlugin.objects.using(db_alias).filter(action_backend='no_storage').update(action_backend='none')


def backward_migration(apps, schema_editor):
    db_alias = schema_editor.connection.alias
    FormPlugin = apps.get_model('aldryn_forms', 'FormPlugin')
    FormPlugin.objects.using(db_alias).filter(action_backend='none').update(action_backend='no_storage')


class Migration(migrations.Migration):

    dependencies = [
        ('aldryn_forms', '0010_auto_20171220_1733'),
    ]

    operations = [
        migrations.RenameField(
            model_name='formplugin',
            old_name='storage_backend',
            new_name='action_backend',
        ),

        migrations.RunPython(forward_migration, backward_migration),

    ]



import re

from django import forms
from django.conf import settings
from django.forms.forms import NON_FIELD_ERRORS
from django.forms.utils import ErrorDict
from django.utils.translation import gettext
from django.utils.translation import gettext_lazy as _

from PIL import Image

from .models import FormPlugin, FormSubmission
from .sizefield.utils import filesizeformat
from .utils import add_form_error, get_action_backends, get_user_model


class FileSizeCheckMixin:

    def __init__(self, *args, **kwargs):
        self.files = []  # This is set in FormPlugin.process_form() in cms_plugins.py
        self.max_size = kwargs.pop('max_size', None)
        self.accepted_types = kwargs.pop('accepted_types', [])
        super().__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        super().clean(*args, **kwargs)

        if not self.files:
            return []

        all_errors = []

        # Check file extension.
        if self.accepted_types:
            accepted_types, main_mimetypes = self.split_mimetypes(self.accepted_types)
            errors = []
            for file_in_memory in self.files:
                match = re.search(r'(\.\w+)$', file_in_memory.name.lower())
                extension = match.group(1) if match else None  # '.csv'
                if not (
                    extension in self.accepted_types
                    or file_in_memory.content_type in self.accepted_types  # noqa: W503 line break before bin operator
                    or file_in_memory.content_type.split("/")[0] in main_mimetypes  # noqa: W503
                ):
                    errors.append(gettext('"%(file_name)s" is not of accepted file type.') % {
                        'file_name': file_in_memory.name})
            if errors:
                all_errors.append(
                    " ".join(errors) + " " + gettext("Accepted file types are") + ": " + ", ".join(
                        self.accepted_types) + ".")

        # Check files size summary.
        if self.max_size is not None:
            errors = []
            files_size_summary = 0
            for file_in_memory in self.files:
                files_size_summary += file_in_memory.size
            if files_size_summary > self.max_size:
                if len(self.files) > 1:
                    msg = gettext('The total file size has exceeded the specified limit %(size)s.')
                else:
                    msg = gettext('File size exceeded the specified limit %(size)s.')
                all_errors.append(msg % {'size': filesizeformat(self.max_size)})

        if all_errors:
            raise forms.ValidationError(" ".join(all_errors))
        return self.files

    def split_mimetypes(self, accepted_types):
        """Split mimetypes with wildcards."""
        # Example of accepted_types: ['.pdf', 'text/plain', 'application/msword', 'image/*', 'text/*']
        accepted, main_mimetypes = [], []
        for name in accepted_types:
            match = re.match(r'(\w+)/\*', name)
            if match:
                main_mimetypes.append(match.group(1))
            else:
                accepted.append(name)
        return accepted, main_mimetypes


class RestrictedFileField(FileSizeCheckMixin, forms.FileField):
    pass


class RestrictedMultipleFilesField(FileSizeCheckMixin, forms.FileField):

    def __init__(self, *args, **kwargs):
        self.max_files = kwargs.pop('max_files', None)
        super().__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        super().clean(*args, **kwargs)
        if not self.files:
            return []
        if self.max_files is not None and len(self.files) > self.max_files:
            raise forms.ValidationError(
                gettext("The number of uploaded files exceeded the set limit of %(limit)s.") % {
                    'limit': self.max_files
                })
        return self.files


class RestrictedImageField(FileSizeCheckMixin, forms.ImageField):

    def __init__(self, *args, **kwargs):
        self.max_width = kwargs.pop('max_width', None)
        self.max_height = kwargs.pop('max_height', None)
        super().__init__(*args, **kwargs)

    def clean(self, *args, **kwargs):
        data = super().clean(*args, **kwargs)

        if data is None or not any([self.max_width, self.max_height]):
            return data

        if hasattr(data, 'image'):
            # Django >= 1.8
            width, height = data.image.size
        else:
            width, height = Image.open(data).size
            # cleanup after ourselves
            data.seek(0)

        if self.max_width and width > self.max_width:
            raise forms.ValidationError(
                gettext(
                    'Image width must be under %(max_size)s pixels. '
                    'Current width is %(actual_size)s pixels.'
                ) % {
                    'max_size': self.max_width,
                    'actual_size': width,
                })

        if self.max_height and height > self.max_height:
            raise forms.ValidationError(
                gettext(
                    'Image height must be under %(max_size)s pixels. '
                    'Current height is %(actual_size)s pixels.'
                ) % {
                    'max_size': self.max_height,
                    'actual_size': height,
                })

        return data


class FormSubmissionBaseForm(forms.Form):

    # these fields are internal.
    # by default we ignore all hidden fields when saving form data to db.
    language = forms.ChoiceField(
        choices=settings.LANGUAGES,
        widget=forms.HiddenInput()
    )
    form_plugin_id = forms.IntegerField(widget=forms.HiddenInput())

    def __init__(self, *args, **kwargs):
        self.form_plugin = kwargs.pop('form_plugin')
        self.request = kwargs.pop('request')
        super().__init__(*args, **kwargs)
        language = self.form_plugin.language

        self.instance = FormSubmission(
            name=self.form_plugin.name,
            language=language,
            form_url=self.request.build_absolute_uri(self.request.path),
        )
        self.fields['language'].initial = language
        self.fields['form_plugin_id'].initial = self.form_plugin.pk

    def _add_error(self, message, field=NON_FIELD_ERRORS):
        try:
            self._errors[field].append(message)
        except (KeyError, TypeError):
            if not self._errors:
                self._errors = ErrorDict()
            self._errors[field] = self.error_class([message])

    def get_serialized_fields(self, is_confirmation=False):
        """
        The `is_confirmation` flag indicates if the data will be used in a
        confirmation email sent to the user submitting the form or if it will be
        used to render the data for the recipients/admin site.
        """
        for field in self.form_plugin.get_form_fields():
            plugin = field.plugin_instance.get_plugin_class_instance()
            # serialize_field can be None or SerializedFormField  namedtuple instance.
            # if None then it means we shouldn't serialize this field.
            serialized_field = plugin.serialize_field(self, field, is_confirmation)

            if serialized_field:
                yield serialized_field

    def get_serialized_field_choices(self, is_confirmation=False):
        """Renders the form data in a format suitable to be serialized.
        """
        fields = self.get_serialized_fields(is_confirmation)
        fields = [(field.label, field.value) for field in fields]
        return fields

    def get_cleaned_data(self, is_confirmation=False):
        fields = self.get_serialized_fields(is_confirmation)
        form_data = {field.name: field.value for field in fields}
        return form_data

    def save(self, commit=False):
        self.instance.set_form_data(self)
        self.instance.save()


class ExtandableErrorForm(forms.ModelForm):

    def append_to_errors(self, field, message):
        add_form_error(form=self, message=message, field=field)


class FormPluginForm(ExtandableErrorForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        if getattr(settings, 'ALDRYN_FORMS_SHOW_ALL_RECIPIENTS', False) and 'recipients' in self.fields:
            self.fields['recipients'].queryset = get_user_model().objects.all()

    def clean(self):
        redirect_type = self.cleaned_data.get('redirect_type')
        redirect_page = self.cleaned_data.get('redirect_page')
        url = self.cleaned_data.get('url')

        if redirect_type:
            if redirect_type == FormPlugin.REDIRECT_TO_PAGE:
                if not redirect_page:
                    self.append_to_errors('redirect_page', _('Please provide CMS page for redirect.'))
                self.cleaned_data['url'] = None

            if redirect_type == FormPlugin.REDIRECT_TO_URL:
                if not url:
                    self.append_to_errors('url', _('Please provide an absolute URL for redirect.'))
                self.cleaned_data['redirect_page'] = None
        else:
            self.cleaned_data['url'] = None
            self.cleaned_data['redirect_page'] = None

        action_backend = get_action_backends().get(self.cleaned_data.get('action_backend'))
        if action_backend is not None:
            error = getattr(action_backend, "clean_form", lambda form: None)(self)
            if error:
                self.append_to_errors('action_backend', error)
        return self.cleaned_data


class BooleanFieldForm(forms.ModelForm):

    def __init__(self, *args, **kwargs):
        if 'instance' not in kwargs:  # creating new one
            initial = kwargs.pop('initial', {})
            initial['required'] = False
            kwargs['initial'] = initial
        super().__init__(*args, **kwargs)

    class Meta:
        fields = ['label', 'help_text', 'required', 'required_message', 'custom_classes']


class SelectFieldForm(forms.ModelForm):

    class Meta:
        fields = ['label', 'help_text', 'required', 'required_message', 'custom_classes']


class RadioFieldForm(forms.ModelForm):

    class Meta:
        fields = ['label', 'help_text', 'required', 'required_message', 'custom_classes']


class CaptchaFieldForm(forms.ModelForm):

    class Meta:
        # captcha is always required
        fields = ['label', 'help_text', 'required_message']


class MinMaxValueForm(ExtandableErrorForm):

    def clean(self):
        min_value = self.cleaned_data.get('min_value')
        max_value = self.cleaned_data.get('max_value')
        if min_value and max_value and min_value > max_value:
            self.append_to_errors('min_value', _('Min value can not be greater than max value.'))
        if self.cleaned_data.get('required') and min_value is not None and min_value < 1:
            self.append_to_errors(
                'min_value', _('If checkbox "Field is required" is set, "Min choices" must be at least 1.'))
        return self.cleaned_data


class TextFieldForm(MinMaxValueForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['min_value'].label = _('Min length')
        self.fields['min_value'].help_text = _('Required number of characters to type.')

        self.fields['max_value'].label = _('Max length')
        self.fields['max_value'].help_text = _('Maximum number of characters to type.')
        self.fields['max_value'].required = False

    class Meta:
        fields = ['label', 'placeholder_text', 'help_text',
                  'min_value', 'max_value', 'required', 'required_message', 'custom_classes']


class HiddenFieldForm(ExtandableErrorForm):
    class Meta:
        fields = ['name', 'initial_value']


class NoRequiredFieldsModelForm(forms.ModelForm):
    """No required fields ModelForm."""

    no_required_fields = []

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        for name in self.no_required_fields:
            self.fields[name].required = False


class DateFieldForm(NoRequiredFieldsModelForm):
    """Date Field Form."""

    no_required_fields = ['earliest_date', 'latest_date']


class DateTimeFieldForm(NoRequiredFieldsModelForm):
    """Datetime Field Form."""

    no_required_fields = ['earliest_datetime', 'latest_datetime']


class TimeFieldForm(NoRequiredFieldsModelForm):
    """Datetime Field Form."""

    no_required_fields = ['earliest_time', 'latest_time']


class EmailFieldForm(TextFieldForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['min_value'].required = False
        self.fields['max_value'].required = False

    class Meta:
        fields = [
            'label',
            'placeholder_text',
            'help_text',
            'min_value',
            'max_value',
            'required',
            'required_message',
            'email_send_notification',
            'email_subject',
            'email_body',
            'custom_classes',
        ]

    def clean(self):
        if "name" in self.changed_data:
            _, action_backend = self.instance.get_parent_form_action_backend()
            if action_backend is not None:
                error = getattr(action_backend, "clean_field", lambda form: None)(self)
                if error:
                    self.append_to_errors('name', error)
        return super().clean()


class FileFieldForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['help_text'].help_text = _(
            'Explanatory text displayed next to input field. Just like this '
            'one. You can use MAXSIZE as a placeholder for the maximum size '
            'configured below.')

    class Meta:
        fields = ['label', 'help_text', 'required', 'required_message',
                  'custom_classes', 'upload_to', 'max_size']


class ImageFieldForm(forms.ModelForm):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['help_text'].help_text = _(
            'Explanatory text displayed next to input field. Just like this '
            'one. You can use MAXSIZE, MAXWIDTH, MAXHEIGHT as a placeholders '
            'for the maximum file size and dimensions configured below.')

    class Meta:
        fields = FileFieldForm.Meta.fields + ['max_height', 'max_width']


class TextAreaFieldForm(TextFieldForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.fields['max_value'].required = False

    class Meta:
        fields = ['label', 'placeholder_text', 'help_text', 'text_area_columns',
                  'text_area_rows', 'min_value', 'max_value', 'required', 'required_message', 'custom_classes']


class MultipleSelectFieldForm(MinMaxValueForm):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.fields['min_value'].label = _('Min choices')
        self.fields['min_value'].help_text = _('Required amount of elements to chose.')

        self.fields['max_value'].label = _('Max choices')
        self.fields['max_value'].help_text = _('Maximum amount of elements to chose.')

    class Meta:
        # 'required' and 'required_message' depend on min_value field validator
        fields = ['label', 'help_text', 'min_value', 'max_value', 'custom_classes']



from email.utils import parseaddr

from django.core.exceptions import ValidationError
from django.core.validators import MaxLengthValidator, MinLengthValidator, validate_email
from django.utils.translation import gettext_lazy as _


def is_valid_recipient(recipient):
    """
    recipient - a string in any of the following formats (RFC 2822):
        name
        useremail@gmail.com
        name <useremail@mail.com>
    """
    if not recipient:
        return False

    email_address = parseaddr(recipient)[1]

    try:
        validate_email(email_address)
    except ValidationError:
        return False
    else:
        return True


class MinChoicesValidator(MinLengthValidator):

    message = _('You have to choose at least %(limit_value)d options (chosen %(show_value)d).')
    code = 'min_choices'


class MaxChoicesValidator(MaxLengthValidator):

    message = _('You can\'t choose more than %(limit_value)d options (chosen %(show_value)d).')
    code = 'max_choices'
